using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;

using Mono.Unix;
 
using Banshee.Base;
using Banshee.Sources;
using Banshee.MediaEngine;
using Banshee.Configuration;

/***************************************************************************
 *  SaveList.cs
 *
 *  Copyright (C) 2005-2006 Ed Neville
 *  Written by Ed Neville, <ed-banshee@_remove_.s5h.net>
 ****************************************************************************/

/*  THIS FILE IS LICENSED UNDER THE MIT LICENSE AS OUTLINED IMMEDIATELY BELOW: 
 *
 *  Permission is hereby granted, free of charge, to any person obtaining a
 *  copy of this software and associated documentation files (the "Software"),  
 *  to deal in the Software without restriction, including without limitation  
 *  the rights to use, copy, modify, merge, publish, distribute, sublicense,  
 *  and/or sell copies of the Software, and to permit persons to whom the  
 *  Software is furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be included in 
 *  all copies or substantial portions of the Software.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR 
 *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
 *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE 
 *  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
 *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
 *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
 *  DEALINGS IN THE SOFTWARE.
 *
 *  $Id: SaveList.cs 81 2007-11-06 22:53:41Z ed $
 *  $URL: svn+ssh://fw/repos/code/mono/Banshee.Plugins.SaveList/SaveList.cs $
 */

/**
 * sqlite db is stored in ~/.config/banshee/banshee.db
 */

public static class PluginModuleEntry {
	public static Type [] GetTypes() {
		return new Type [] {
			typeof( Banshee.Plugins.SaveList.SaveListPlugin )
		};
	}
}

namespace Banshee.Plugins.SaveList {
	public class SaveListPlugin : Banshee.Plugins.Plugin {
		protected override string ConfigurationName { 
			get { 
				return "SaveList"; 
			} 
		}
		
		public override string DisplayName { get { return "SaveList"; } }
		
		public override string Description {
			get {
				return Catalog.GetString(
					"A small plugin that saves the " +
					"current song to resume on " +
					"startup" );
			}
		}
		
		public override string [] Authors {
			get {
				return new string [] {
					"Ed Neville"
				};
			}
		}
 
	// --------------------------------------------------------------- //

		private int last_played_track_id = -1;
		private uint timeout_id;
		private SaveListSource source;

		public IEnumerable get_played_list() {
			IDataReader r = Globals.Library.Db.Query( 
				"select Tracks_TrackId from LastPlayed" );

			while( r.Read() ) {
				yield return r["Tracks_TrackId"];
			}
			r.Dispose();
		}

		public void load_playlist() {
			foreach( int id in this.get_played_list() ) {
				source.AddTrack( Globals.Library.GetTrack( id ) );
			}
		}
	
		protected override void PluginInitialize() {
			Console.WriteLine( "Initializing SaveList Plugin" );
			source = new SaveListSource();
			SourceManager.AddSource( source );

			/**
			 * now lets create a table to hold our single value.
			 * could use the date, but that might be a lengthy sort
			 * operation for the end user. lets not make the user
			 * sort their tracks eh, and lets not sort them our
			 * selves either.
			 */
			string statement = "create table if not exists LastPlayed " + 
				"( Tracks_TrackId integer );";

			Globals.Library.Db.QuerySingle( statement );

			load_playlist();

			PlayerEngineCore.EventChanged += 
				OnPlayerEngineEventChanged;

		}
	
		// optional, this is a virtual override.
		// only provide an implementation if there are
		// resources to be disposed of or other objects
		// that need notification, etc.
		protected override void PluginDispose() {
			Console.WriteLine("Disposing SaveList Plugin");
	
			/* GLib.Source.Remove( this.timeout_id );
			this.timeout_id = 0; */
		
			/**
			 * remove the database table as the user doesn't want
			 * this plugin anymore
			 */

			/*
			string statement = "drop table if exists LastPlayed;";
			Globals.Library.Db.QuerySingle( statement );
			*/
			
			PlayerEngineCore.EventChanged -= 
				OnPlayerEngineEventChanged;

			SourceManager.RemoveSource( source );
		}
	
		// optional, this is a virtual override.
		// only provide an implementation if there
		// is a configuration widget to show
		private Gtk.Button button = new Gtk.Button("button");
	
		public override Gtk.Widget GetConfigurationWidget() {
			return button;
		}
		
		private void update_track_info( int track_id ) {
			/**
			 * first we must remove excess tracks from the
			 * database table, then we will insert this to
			 * top of the stack
			 */
			string sql = "insert into LastPlayed (Tracks_TrackId) values ( " + track_id + " )";
			Globals.Library.Db.QuerySingle( sql );
		}
		
		private void OnPlayerEngineEventChanged( object o,
			PlayerEngineEventArgs args ) {

				switch( args.Event ) {
					case PlayerEngineEvent.TrackInfoUpdated: {
						int track_id = PlayerEngineCore.CurrentTrack.TrackId;
						update_track_info( track_id );
						source.AddTrack( Globals.Library.GetTrack( track_id ) );

						break;
					}
					
				}
			}

	
		public class SaveListSource : Banshee.Sources.Source {
		
			private Gtk.Button show_tracks = new Gtk.Button("Change View");
			private List<TrackInfo> tracks = new List<TrackInfo>();
			
			
			public SaveListSource() : base("SaveList Source", 150) {
				show_tracks.Show();
				show_tracks.Clicked += 
					delegate( object o, EventArgs args ) {
					show_tracks = null;
					OnViewChanged();
				};
			}
		
			public override void AddTrack(TrackInfo track) {
				tracks.Add( track );
				base.OnTrackAdded( track );
				OnUpdated();
			}	
		
			public override IEnumerable<TrackInfo> Tracks {
			get { return tracks; }
			}
			
			public override int Count {
			get { return tracks.Count; }
			}
		
			public override Gtk.Widget ViewWidget {
				get { return show_tracks; }
			}

		}
	}
}
